"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const socket_io_client_1 = __importDefault(require("socket.io-client"));
const jquery_1 = __importDefault(require("jquery"));
const epic_chat_helper_1 = require("epic-chat-helper");
class epicChatHelper {
    // Constructor
    constructor() {
        // Defaults
        this.url = "";
        this.socket = null;
        this.currentContact = {
            id: "",
            name: null,
            contact: null,
            reference: "",
            address: null,
            trace: null,
            meta: null,
        };
        this.contactList = [];
        this.connectionCount = 0;
        this.connections = {};
        this.me = null;
        this.eventPrefix = "epicChatHelper.";
        this.traceUrl = "http://www.geoplugin.net/json.gp";
        this.traceHandler = (traceData) => traceData;
        this.trace = null;
        // Html Template
        this.templateVariables = {};
        this.parentElement = null;
        this.childTemplate = {};
        this.childTemplateProcessed = {};
        this.childElements = {};
        this.templateRendered = false;
        this.appends = [];
        this.init = (contactList, uri, connectOptions) => {
            this.contactList = contactList;
            this.socket = socket_io_client_1.default.connect(uri, connectOptions);
            return this;
        };
        this.connect = (connected, connectionError) => {
            var _a;
            (_a = this.socket) === null || _a === void 0 ? void 0 : _a.on("connect", () => {
                if (connected)
                    connected();
            }).on("connect_error", (error) => {
                if (connectionError)
                    connectionError(error);
            }).on('connect_timeout', (error) => {
                if (connectionError)
                    connectionError(error);
            }).on('error', (error) => {
                if (connectionError)
                    connectionError(error);
            });
            return this;
        };
        this.traceConfig = (traceUrl, traceHandler) => {
            this.traceUrl = traceUrl;
            this.traceHandler = traceHandler;
            return this;
        };
        this.authenticate = (reference, name, contact, meta = null) => {
            return new Promise((resolve, reject) => {
                let authenticate = () => {
                    var _a, _b, _c;
                    // Resolve Reference
                    if (!reference)
                        reference = (new Date()).getTime() + Math.random().toString(36).substr(2, 9);
                    // Create Authentication Object
                    let authentication = {
                        reference: reference,
                        name: name,
                        contact: contact,
                        trace: this.trace,
                        meta: meta,
                    };
                    // Authenticate
                    (_a = this.socket) === null || _a === void 0 ? void 0 : _a.emit(this.eventPrefix + "authenticate", authentication);
                    // Authenticated
                    (_b = this.socket) === null || _b === void 0 ? void 0 : _b.on(this.eventPrefix + "connected", (connection) => {
                        this.me = connection.connections[this.socket.id];
                        delete connection.connections[this.socket.id];
                        this.connections = connection.connections;
                        this.connectionCount = connection.connectionCount;
                        if (typeof this.connectionAdded == "function")
                            this.connectionAdded(connection);
                        if (typeof this.connectionsUpdate == "function")
                            this.connectionsUpdate(connection);
                        if (connection.connection.reference == reference)
                            resolve();
                    });
                    // Disconnected
                    (_c = this.socket) === null || _c === void 0 ? void 0 : _c.on(this.eventPrefix + "disconnected", (connection) => {
                        this.connections = connection.connections;
                        this.connectionCount = connection.connectionCount;
                        if (typeof this.connectionRemoved == "function")
                            this.connectionRemoved(connection);
                        if (typeof this.connectionsUpdate == "function")
                            this.connectionsUpdate(connection);
                    });
                };
                // Connecting First Time
                if (this.socket && this.socket.connected)
                    // Trace User
                    if (!this.trace)
                        jquery_1.default.ajax({
                            dataType: "json",
                            url: this.traceUrl,
                            success: (data) => this.trace = this.traceHandler(data),
                            complete: () => authenticate()
                        });
                    else
                        authenticate();
                else
                    reject();
            });
        };
        this.sendMessage = (message, reference) => {
            let contact = null;
            if (typeof reference != "undefined")
                if (this.contactList.includes(reference))
                    contact = this.getContact(reference);
                else
                    throw new Error("Contact Not Found!");
            else
                contact = this.currentContact;
            //Emit Message
            if (this.socket)
                this.socket.emit(this.eventPrefix + "message", {
                    receiver: contact,
                    message: message,
                });
            else
                throw new Error("Socket Client Is Not Available!");
            return this;
        };
        this.receiveMessage = (callback) => {
            if (this.currentContact != null) {
                if (this.socket)
                    this.socket.on(this.eventPrefix + "message", (message) => {
                        var _a;
                        if (message.sender.id == ((_a = this.currentContact) === null || _a === void 0 ? void 0 : _a.id))
                            callback(message);
                    });
            }
            else
                throw new Error("No Contact Has Been Selected!");
            return this;
        };
        this.typing = (text) => {
            return this.sendEvent({
                event: "typing",
                data: { text: text, }
            }, epic_chat_helper_1.EVENT_TYPE.UNICAST, this.getCurrentContact());
        };
        this.onTyping = (callback) => {
            return this.receiveEvent((event) => {
                if (typeof event.payload.event != "undefined" && event.payload.event == "typing")
                    callback(event.payload.data.text);
            });
        };
        this.sendEvent = (payload, type = epic_chat_helper_1.EVENT_TYPE.BROADCAST, receiver) => {
            if (this.me != null) {
                let event = {
                    type: type,
                    sender: this.me,
                    receiver: receiver,
                    payload: payload
                };
                if (this.socket)
                    this.socket.emit(this.eventPrefix + "send_event", event);
                else
                    throw new Error("Socket Client Is Not Available!");
            }
            else
                throw new Error("Error Occured! User Not Authenticated Yet!");
            return this;
        };
        this.receiveEvent = (callback) => {
            if (this.socket)
                this.socket.on(this.eventPrefix + "event", (data) => {
                    callback(data);
                });
            return this;
        };
        this.addContact = (reference) => {
            if (!this.contactList.includes(reference))
                this.contactList.push(reference);
            return this;
        };
        this.setContact = (reference) => {
            if (this.contactList.includes(reference))
                this.currentContact = this.getContact(reference);
            else
                throw new Error("Contact Not Found!");
            return this;
        };
        this.getContact = (reference) => {
            let contact = null;
            for (let i in this.connections) {
                if (this.connections.hasOwnProperty(i)) {
                    if (this.connections[i].reference == reference)
                        contact = this.connections[i];
                }
            }
            if (contact == null)
                throw new Error("Contact Not Available!");
            else
                return contact;
        };
        this.getCurrentContact = () => {
            return this.currentContact;
        };
        this.templateSetParent = (htmlSelector) => {
            this.parentElement = jquery_1.default(htmlSelector)[0];
            return this;
        };
        this.templateGetParent = () => {
            if (this.parentElement)
                return this.parentElement;
            else
                throw new Error("Parent Element Not Found!");
        };
        this.templateSelect = (elementName, htmlSelector, htmlTemplateString = "", templateVariables = {}) => {
            if (this.parentElement) {
                this.childElements[elementName] = jquery_1.default(this.parentElement).find(htmlSelector)[0];
                this.childTemplate[elementName] = htmlTemplateString;
                this.updateTemplateVariables(templateVariables);
            }
            else
                throw new Error("Parent Element Is Not Selected Yet!");
            return this;
        };
        this.templateAppend = (appendId, htmlSelector, htmlTemplateString = "", templateVariables = {}, render = false, rendered) => {
            this.appends.push({
                id: appendId,
                htmlSelector: htmlSelector,
                htmlTemplateString: htmlTemplateString
            });
            this.updateTemplateVariables(templateVariables, render, rendered);
            return this;
        };
        this.removeAppendById = (appendId) => {
            this.appends = this.appends.filter((v) => {
                return v.id !== appendId;
            });
            return this;
        };
        this.removeAppendByParent = (htmlSelector) => {
            this.appends = this.appends.filter((v) => {
                return v.htmlSelector !== htmlSelector;
            });
            return this;
        };
        this.clearAppend = () => {
            this.appends = [];
            return this;
        };
        this.putVariables = (template) => {
            let match, matches = [];
            let regex = new RegExp('\\{\\{\\s*(.+?)\\s*\\}\\}', "g");
            while (match = regex.exec(template))
                matches.push(match[1]);
            let result = template;
            if (matches.length)
                matches.forEach((v) => {
                    if (typeof this.templateVariables[v] != "undefined" && this.templateVariables[v] !== null)
                        // @ts-ignore
                        result = result.replace((new RegExp('\\{\\{\\s*(' + v + ')\\s*\\}\\}', "g")), this.templateVariables[v]);
                });
            return result;
        };
        this.processTemplate = () => {
            for (let e in this.childTemplate)
                if (this.childTemplate.hasOwnProperty(e))
                    this.childTemplateProcessed[e] = this.putVariables(this.childTemplate[e]);
            return this;
        };
        this.renderTemplate = (rendered, rendering) => {
            this.processTemplate();
            for (let t in this.childTemplateProcessed)
                if (this.childTemplateProcessed.hasOwnProperty(t) && typeof this.childElements[t] != "undefined") {
                    jquery_1.default(this.childElements[t]).html(this.childTemplateProcessed[t]);
                    if (rendering)
                        rendering(this.childElements[t]);
                }
            this.templateRendered = true;
            if (rendered)
                rendered(this.parentElement);
            return this;
        };
        this.renderAppend = (renderLast = false, clear = false) => {
            if (this.templateRendered) {
                if (!renderLast) {
                    this.appends.forEach((v) => {
                        jquery_1.default(this.parentElement).find(v.htmlSelector)
                            .append(this.putVariables(v.htmlTemplateString));
                    });
                }
                else {
                    let element = jquery_1.default(this.parentElement).find(this.appends[this.appends.length - 1].htmlSelector);
                    if (clear)
                        element.empty();
                    element.append(this.putVariables(this.appends[this.appends.length - 1].htmlTemplateString));
                }
            }
            else
                throw new Error("Template Is Not Rendered Yet!");
            return this;
        };
        this.updateTemplateVariables = (templateVariables, render = false, rendered) => {
            for (let k in templateVariables)
                if (templateVariables.hasOwnProperty(k))
                    this.templateVariables[k] = templateVariables[k];
            //Render If Needed.
            if (render)
                this.renderTemplate(rendered);
            return this;
        };
        return this;
    }
}
exports.epicChatHelper = epicChatHelper;
//# sourceMappingURL=epicChatHelperClient.js.map